{================================================================================
Copyright (C) 1997-2003 Mills Enterprise

Unit     : OpenOfficePDF
Purpose  : This unit provides a single class to enabled the programmer to create a PDF
          document through the OpenOffice.Org API.  This unit is provided as freeware
          please use at your own risk.  This file can be found as a part of the rmControls
          library
Date     : 07-25-2005
Author   : Ryan J. Mills
Version  : 1.93
================================================================================}

unit OOPDF;

{$ifdef fpc}
{$mode objfpc}
{$H+}
{$endif}

interface

uses ComObj, Variants, sysutils;

type
  TOOoWriter = class(TObject)
  private
    fOpenOffice : Variant;
    fDocument : Variant;
    fConnected : boolean;
    fDocumentOpened : boolean;
    fDesktop : Variant;
    fHTMLSrc : boolean;

    function MakePropertyValue(PropName:string; PropValue:variant):variant;
  public
     Constructor Create;
     destructor Destroy; override;

     function Connect : boolean;
     procedure Disconnect;

     function OpenDocument(Filename:string):boolean;
     procedure SaveToPDF(FileName:string);
     procedure CloseDocument;
  end;

implementation

uses
   ActiveX;

{ TOOoWriter }

procedure TOOoWriter.CloseDocument;
begin
    if fDocumentOpened then
    begin
       fDocument.Close(false);

       fDocumentOpened := false;
       fDocument := Unassigned;

       fDesktop.Terminate;
       fDesktop := UnAssigned;
    end;
end;

function TOOoWriter.Connect: boolean;
begin
    if  VarIsEmpty(fOpenOffice) then
        fOpenOffice := CreateOleObject('com.sun.star.ServiceManager');

    fConnected := not (VarIsEmpty(fOpenOffice) or VarIsNull(fOpenOffice));
    Result := fConnected;
end;

constructor TOOoWriter.Create;
begin
  inherited;
  CoInitialize(nil);
end;

destructor TOOoWriter.Destroy;
begin
  CoUninitialize;
  inherited;
end;

procedure TOOoWriter.Disconnect;
begin
    if fDocumentOpened then
       CloseDocument;

    fConnected := false;
    fOpenOffice := Unassigned;
end;

function TOOoWriter.MakePropertyValue(PropName: string;
  PropValue: variant): variant;
var
   Struct: variant;
begin
    Struct := fOpenOffice.Bridge_GetStruct('com.sun.star.beans.PropertyValue');
    Struct.Name := PropName;
    Struct.Value := PropValue;
    Result := Struct;
end;

function TOOoWriter.OpenDocument(Filename: string): boolean;
var
   wProperties : Variant;
   wViewSettings : Variant;
   wController : Variant;
begin
   if not fConnected then
      abort;

   fDesktop := fOpenOffice.createInstance('com.sun.star.frame.Desktop');

   wProperties := VarArrayCreate([0, 0], varVariant);
   wProperties[0] := MakePropertyValue('Hidden', True);

   fDocument := fDesktop.loadComponentFromURL('file:///'+ StringReplace(FileName, '\', '/', [rfIgnoreCase, rfReplaceAll]) , '_blank', 0, wProperties);

   fDocumentOpened := not (VarIsEmpty(fDocument) or VarIsNull(fDocument));

   fHTMLSrc := pos('.htm', lowercase(extractfileext(Filename))) > 0;

   if fDocumentOpened and fHTMLSrc then
   begin
     wController := fDocument.Getcurrentcontroller;
     if not (VarIsEmpty(wController) or VarIsNull(wController)) then
     begin
        wViewSettings := wController.getviewsettings;
        if not (VarIsEmpty(wViewSettings) or VarIsNull(wViewSettings)) then
           wViewSettings.ShowOnlineLayout := false;
     end;
     wViewSettings := Unassigned;
     wController := Unassigned;
   end;

   result := fDocumentOpened;
end;

procedure TOOoWriter.SaveToPDF(FileName: string);
var
   wProperties: variant;

begin
   if not (fConnected and fDocumentOpened) then
      abort;

   wProperties := VarArrayCreate([0, 3], varVariant);
   if fHTMLSrc then
      wProperties[0] := MakePropertyValue('FilterName', 'writer_web_pdf_Export')
   else
      wProperties[0] := MakePropertyValue('FilterName', 'writer_pdf_Export');

   wProperties[1] := MakePropertyValue('CompressionMode', '1');
   wProperties[2] := MakePropertyValue('Pages', 'All');
   wProperties[3] := MakePropertyValue('Overwrite', TRUE);

   fDocument.StoreToURL('file:///'+ StringReplace(FileName, '\', '/', [rfIgnoreCase, rfReplaceAll]), wProperties);
end;

end.